import * as React from 'react';
import Table from '@mui/material/Table';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import TableHead from '@mui/material/TableHead';
import TableRow from '@mui/material/TableRow';
import Title from './Title';
import axios from 'axios';
import { useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux'
import { fetchSpotPrices } from '../../store/spotPrice';
import {ThunkDispatch} from "@reduxjs/toolkit";
import {fetchFuturesPrices} from "../../store/futuresPrice";

interface FuturesPosition {
    symbol: string;
    positionAmt: string;
    entryPrice: string;
    breakEvenPrice: string;
    markPrice: string;
    unRealizedProfit: string;
    liquidationPrice: string;
    leverage: string;
    maxNotionalValue: string;
    marginType: string;
    isolatedMargin: string;
    isAutoAddMargin: string;
    positionSide: string;
    notional: string;
    isolatedWallet: string;
    updateTime: number;
    isolated: boolean;
    adlQuantile: number;
}

interface FuturesPrice {
    symbol: string;
    bidPrice: string;
    bidQty: string;
    askPrice: string;
    askQty: string;
    time: number;
    lastUpdateId: number;
}

interface SpotPosition {
    asset: string;
    free: string;
    locked: string;
    borrowed: string;
    interest: string;
    netAsset: string;
}

interface SpotPrice {
    symbol: string;
    bidPrice: string;
    bidQty: string;
    askPrice: string;
    askQty: string;
}

interface FundingRate {
    symbol: string;
    lastFundingRate: string;
    interestRate: string;
    nextFundingTime: number;
    time: number;
}

interface MergedRow {
    symbol: string;
    futureBidPrice: string;
    futureBidQty: string;
    futureAskPrice: string;
    futureAskQty: string;
    spotBidPrice: string;
    spotBidQty: string;
    spotAskPrice: string;
    spotAskQty: string;
    spotTotalAmount: string;
    positionAmt: string;
    entryPrice: string;
    breakEvenPrice: string;
    markPrice: string;
    unRealizedProfit: string;
    liquidationPrice: string;
    leverage: string;
    maxNotionalValue: string;
    marginType: string;
    isolatedMargin: string;
    isAutoAddMargin: string;
    positionSide: string;
    notional: string;
    isolatedWallet: string;
    updateTime: number;
    isolated: boolean;
    adlQuantile: number;
    lastFundingRate: any;
    interestRate: any;
    nextFundingTime: any;
}

export default function FuturesPositions() {
    const spotPriceStore = useSelector((state: any) => state.spotPrice)
    const futuresPriceStore = useSelector((state: any) => state.futuresPrice)
    const dispatch = useDispatch<ThunkDispatch<any, any, any>>()

    const [rows, setRows] = useState<MergedRow[]>([]);

    useEffect(() => {
        const fetchData = async () => {
            dispatch(fetchSpotPrices());
            dispatch(fetchFuturesPrices());
        };

        fetchData();

        const intervalId = setInterval(fetchData, 5000);

        return () => clearInterval(intervalId);
    }, [dispatch]);

    useEffect(() => {
        const fetchData = async () => {
            try {
                const [futuresPositionResponse, spotPositionResponse, fundingRateResponse] = await Promise.all([
                    axios.get('http://localhost:4000/futures/myPosition'),
                    axios.get('http://localhost:4000/spot/myPosition'),
                    axios.get('http://localhost:4000/futures/fundingRate?page=1&pageSize=1000')
                ]);

                const [futuresPriceResponse, spotPriceResponse] = [futuresPriceStore, spotPriceStore];

                const futuresPositions: FuturesPosition[] = futuresPositionResponse.data;
                const futuresPrices: FuturesPrice[] = futuresPriceResponse.data;
                const spotPositions: SpotPosition[] = spotPositionResponse.data;
                const spotPrices: SpotPrice[] = spotPriceResponse.data;
                const fundingRates: FundingRate[] = fundingRateResponse.data;

                // Merge data
                const mergedRows: MergedRow[] = futuresPositions.map((futuresPosition) => {
                    const matchingFuturePrice = futuresPrices.find((price) => price.symbol === futuresPosition.symbol);
                    const matchingSpotPosition = spotPositions.find((position) => position.asset === futuresPosition.symbol.slice(0, -4));
                    const matchingSpotPrice = spotPrices.find((price) => price.symbol === futuresPosition.symbol);
                    const matchingFundingRate = fundingRates.find((rate) => rate.symbol === futuresPosition.symbol);

                    return {
                        symbol: futuresPosition.symbol,
                        futureBidPrice: matchingFuturePrice ? matchingFuturePrice.bidPrice : '',
                        futureBidQty: matchingFuturePrice ? matchingFuturePrice.bidQty : '',
                        futureAskPrice: matchingFuturePrice ? matchingFuturePrice.askPrice : '',
                        futureAskQty: matchingFuturePrice ? matchingFuturePrice.askQty : '',
                        spotBidPrice: matchingSpotPrice ? matchingSpotPrice.bidPrice : '',
                        spotBidQty: matchingSpotPrice ? matchingSpotPrice.bidQty : '',
                        spotAskPrice: matchingSpotPrice ? matchingSpotPrice.askPrice : '',
                        spotAskQty: matchingSpotPrice ? matchingSpotPrice.askQty : '',
                        spotTotalAmount: matchingSpotPosition ? matchingSpotPosition.free + matchingSpotPosition.locked : '',
                        positionAmt: futuresPosition.positionAmt,
                        entryPrice: futuresPosition.entryPrice,
                        breakEvenPrice: futuresPosition.breakEvenPrice,
                        markPrice: futuresPosition.markPrice,
                        unRealizedProfit: futuresPosition.unRealizedProfit,
                        liquidationPrice: futuresPosition.liquidationPrice,
                        leverage: futuresPosition.leverage,
                        maxNotionalValue: futuresPosition.maxNotionalValue,
                        marginType: futuresPosition.marginType,
                        isolatedMargin: futuresPosition.isolatedMargin,
                        isAutoAddMargin: futuresPosition.isAutoAddMargin,
                        positionSide: futuresPosition.positionSide,
                        notional: futuresPosition.notional,
                        isolatedWallet: futuresPosition.isolatedWallet,
                        updateTime: matchingFuturePrice ? matchingFuturePrice.time : 0,
                        isolated: futuresPosition.isolated,
                        adlQuantile: futuresPosition.adlQuantile,
                        lastFundingRate: matchingFundingRate ? matchingFundingRate.lastFundingRate : '',
                        interestRate: matchingFundingRate ? matchingFundingRate.interestRate : '',
                        nextFundingTime: matchingFundingRate ? matchingFundingRate.nextFundingTime : '',
                    };
                });

                setRows(mergedRows);
            } catch (error) {
                console.error('Error fetching data:', error);
            }
        };

        fetchData();

    }, [spotPriceStore, futuresPriceStore]);

    const formatTime = (timestamp: number) => {
        const date = new Date(timestamp);
        const day = date.getDate().toString().padStart(2, '0');
        const month = (date.getMonth() + 1).toString().padStart(2, '0');
        const year = date.getFullYear();
        const hours = date.getHours().toString().padStart(2, '0');
        const minutes = date.getMinutes().toString().padStart(2, '0');
        const seconds = date.getSeconds().toString().padStart(2, '0');
        const meridian = Number(hours) >= 12 ? 'PM' : 'AM';
        const formattedDate = `${Number(hours) % 12 || 12}:${minutes}:${seconds} ${meridian} @ ${day}/${month}/${year}`;
        return formattedDate;
    };

    return (
        <>
            <Title>Futures Positions</Title>
            <Table size="small">
                <TableHead>
                    <TableRow>
                        <TableCell>Symbol</TableCell>
                        <TableCell>% to Liquidation</TableCell>
                        <TableCell>Funding Rate</TableCell>
                        {/* <TableCell>Future Bid Price</TableCell>
                        <TableCell>Future Bid Qty</TableCell>*/}
                        {/*<TableCell>Future Ask Qty</TableCell>
                        <TableCell>Spot Bid Price</TableCell>
                        <TableCell>Spot Bid Qty</TableCell>
                        <TableCell>Spot Ask Price</TableCell>
                        <TableCell>Spot Ask Qty</TableCell> */}
                        <TableCell>Spot USD</TableCell>
                        <TableCell>Futures USD</TableCell>
                        <TableCell>Spot Unit</TableCell>
                        <TableCell>Futures Unit</TableCell>
                        {/*<TableCell>Entry Price</TableCell>
                        <TableCell>Break Even Price</TableCell>
                        <TableCell>Mark Price</TableCell>
                        <TableCell>Unrealized Profit</TableCell>*/}
                        <TableCell>Future Ask Price</TableCell>
                        <TableCell>Liquidation Price</TableCell>
                        <TableCell>Fut Bid / Spot Ask</TableCell>
                        {/*<TableCell>Leverage</TableCell>
                        <TableCell>Max Notional Value</TableCell>
                        <TableCell>Margin Type</TableCell>
                        <TableCell>Isolated Margin</TableCell>
                        <TableCell>Auto Add Margin</TableCell>
                        <TableCell>Position Side</TableCell>
                        <TableCell>Notional</TableCell>
                        <TableCell>Isolated Wallet</TableCell>*/}
                        <TableCell>Update Time</TableCell>
                        {/*<TableCell>Isolated</TableCell>
                        <TableCell>ADL Quantile</TableCell>*/}
                    </TableRow>
                </TableHead>
                <TableBody>
                    {rows.map((row, index) => (
                        <TableRow key={index}>
                        <TableCell>{row.symbol}</TableCell>
                        <TableCell>{((Number(row.liquidationPrice)-Number(row.futureAskPrice))*100/Number(row.futureAskPrice)).toLocaleString()}%</TableCell>
                        <TableCell>{(row.lastFundingRate * 100).toLocaleString()}%</TableCell>
                        {/* <TableCell>{row.futureBidPrice}</TableCell>
                        <TableCell>{row.futureBidQty}</TableCell>*/}
                        {/*<TableCell>{row.futureAskQty}</TableCell>
                        <TableCell>{row.spotBidPrice}</TableCell>
                        <TableCell>{row.spotBidQty}</TableCell>
                        <TableCell>{row.spotAskPrice}</TableCell>
                        <TableCell>{row.spotAskQty}</TableCell> */}
                        <TableCell>{Number(Number(row.spotTotalAmount) * Number(row.spotAskPrice)).toFixed(4)}</TableCell>
                        <TableCell>{Number(Number(row.positionAmt) * Number(row.futureAskPrice)).toFixed(4)}</TableCell>
                        <TableCell>{Number(Number(row.spotTotalAmount)).toFixed(4)}</TableCell>
                        <TableCell>{Number(Number(row.positionAmt)).toFixed(4)}</TableCell>
                        {/*<TableCell>{row.entryPrice}</TableCell>
                        <TableCell>{row.breakEvenPrice}</TableCell>
                        <TableCell>{row.markPrice}</TableCell>
                        <TableCell>{row.unRealizedProfit}</TableCell>*/}
                        <TableCell>{Number(Number(row.futureAskPrice)).toFixed(4)}</TableCell>
                        <TableCell>{Number(row.liquidationPrice).toFixed(4)}</TableCell>
                        <TableCell>{(((Number(row.futureBidPrice)/Number(row.spotAskPrice))-1)*100).toLocaleString()}%</TableCell>
                        {/*<TableCell>{row.leverage}</TableCell>
                        <TableCell>{row.maxNotionalValue}</TableCell>
                        <TableCell>{row.marginType}</TableCell>
                        <TableCell>{row.isolatedMargin}</TableCell>
                        <TableCell>{row.isAutoAddMargin}</TableCell>
                        <TableCell>{row.positionSide}</TableCell>
                        <TableCell>{row.notional}</TableCell>
                        <TableCell>{row.isolatedWallet}</TableCell>*/}
                        <TableCell>{formatTime(row.updateTime)}</TableCell>
                        {/*<TableCell>{row.isolated ? 'Yes' : 'No'}</TableCell>
                        <TableCell>{row.adlQuantile}</TableCell>*/}
                    </TableRow>
                    ))}
                </TableBody>
            </Table>
        </>
    );
}
