import { Box, Button, Card, CardContent, Grid, Modal, TextField, Typography } from '@mui/material'
import React, { useEffect, useState } from 'react'
import DashboardLayout from '../../components/dashboard/Layout'
import RequireLogin from '../../guards/RequireLogin'
import { useAuth } from '../../contexts/AuthContext'
import LoadingButton from "@mui/lab/LoadingButton";
import { FieldError, FormError } from '..'
import { axiosPrivate } from '../../api/axios'
import validator from 'validator'
import { AxiosError } from 'axios'
import { useRouter } from 'next/navigation'

export interface APIKeyType {
  id: number;
  title: string;
  key: string;
  secret: string;
  createdAt: string;
  updatedAt: string;
}

const modalStyle = {
  position: "absolute" as "absolute",
  top: "50%",
  left: "50%",
  transform: "translate(-50%, -50%)",
  width: 400,
  bgcolor: "background.paper",
  border: "2px solid #000",
  boxShadow: 24,
  p: 4,
};

export interface ApiKeyForm {
  id: number;
  title: string;
  key: string;
  secret: string;
}

const Dashboard = () => {
  const router = useRouter();
  const { token, userInfo, setNavTitle, logout } = useAuth();
  const [loading, setLoading] = React.useState(false);
  const [errors, setErrors] = useState<Array<FormError>>([]);
  const [openModal, setOpenModal] = React.useState(false);
  
  useEffect(() => {
    setNavTitle('Dashboard');
  },[])

  const getFieldError = (field: string): FieldError => {
    const find = errors.find((item) => item.field == field);
    if (find == undefined) {
      return {
        valid: false,
        text: "",
      };
    }
    return {
      valid: true,
      text: find.error,
    };
  };

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    setErrors([]);
    const data = new FormData(event.currentTarget);
    let newErrors: Array<FormError> = [];

    if (validator.isEmpty(data.get("new")!.toString())) {
      newErrors.push({ field: "new", error: "Required field" });
    }
    if (validator.isEmpty(data.get("old")!.toString())) {
      newErrors.push({ field: "old", error: "Required field" });
    }


    setErrors(newErrors)
    if (newErrors.length === 0) {
      try {
        setLoading(true);
          const response = await axiosPrivate(token!).patch(
            "/api/changePassword",
            JSON.stringify({
              new: data.get("new")?.toString(),
              old: data.get("old")?.toString()
            }),
            {
              headers: {
                "Content-Type": "application/json",
              },
            }
          );



        setLoading(false);
        setOpenModal(false);
        // Logout
        logout();
        router.push('/');

      } catch (e) {
        setLoading(false);
        const axiosError = e as AxiosError;
        if (axiosError.response) {
          const response = axiosError.response.data as any;
          if (response["error"] == "MIS_MATCHED_PASSWORD") {
            setErrors((prevErrors) => [
              ...prevErrors,
              { field: "old", error: "Mis-Matched" },
            ]);
          }

        }
      }
    }
  };
  
  return (
    <RequireLogin>
    <DashboardLayout>
      <Grid container spacing={3}>
        {/* Main Content */}
        <Grid item xs={12}>
          {/* <Paper sx={{ p: 2, display: "flex", flexDirection: "column" }}> */}
            <Box sx={{ display: 'flex', justifyContent: 'center', alignItems: 'center', backgroundColor: '#f5f5f5',
          // minHeight: '100vh',
            }}>
              <Card sx={{ width: 400, padding: 2 }}>
                <CardContent>
                  <Typography variant="h5" component="div" sx={{ textAlign: 'center', marginBottom: 2 }}>
                    {userInfo?.user.firstName} {userInfo?.user.lastName}
                  </Typography>
                  <Typography variant="body2" color="text.secondary" sx={{ textAlign: 'center', marginBottom: 2 }}>
                    @{userInfo?.user.username}
                  </Typography>
                  <Typography variant="body2" color="text.secondary" sx={{ textAlign: 'center', marginBottom: 2 }}>
                    {userInfo?.user.email}
                  </Typography>
                  <Typography variant="body2" color="text.secondary" sx={{ textAlign: 'center', marginBottom: 2 }}>
                    Member since: {new Date(userInfo?.user.createdAt).toLocaleDateString()}
                  </Typography>
                  <Grid container justifyContent={'center'} sx={{marginTop: 4}} >
                    <Button variant="contained" onClick={() => setOpenModal(true)}>Change Password</Button>
                  </Grid>
                </CardContent>
              </Card>
            </Box>
          {/* </Paper> */}
        </Grid>
      </Grid>

      <Modal
        open={openModal}
        onClose={() => {
          setErrors([]);
          setOpenModal(false);
        }}
        aria-labelledby="modal-modal-title"
        aria-describedby="modal-modal-description"
      >
        <Box sx={modalStyle}>
          <Box
            component="form"
            onSubmit={handleSubmit}
            noValidate
            sx={{ mt: 1 }}
          >

          <TextField
                  error={getFieldError("new").valid}
                  margin="normal"
                  required
                  fullWidth
                  id="new"
                  label="New password"
                  name="new"
                  autoComplete="new"
                  autoFocus
                  helperText={getFieldError("new").text}
                />
                <TextField
                  error={getFieldError("old").valid}
                  margin="normal"
                  required
                  fullWidth
                  name="old"
                  label="Old password"
                  id="old"
                  autoComplete="old"
                  helperText={getFieldError("old").text}
                />


            <LoadingButton
              loading={loading}
              type="submit"
              fullWidth
              variant="contained"
              sx={{ mt: 3, mb: 2 }}
            >
              Change
            </LoadingButton>
          </Box>
        </Box>
      </Modal>
    </DashboardLayout>
  </RequireLogin>
  )
}

export default Dashboard