"use client";
import React, { useEffect, useState } from "react";
import RequireLogin from "../../guards/RequireLogin";

import {
  Box,
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Grid,
  Modal,
  Paper,
  TextField,
  Typography,
} from "@mui/material";
import DashboardLayout from "../../components/dashboard/Layout";
import { DataGrid, GridColDef } from "@mui/x-data-grid";
import axios, { AxiosError } from "axios";
import { axiosPrivate } from "../../api/axios";
import { useAuth } from "../../contexts/AuthContext";
import ModeEditOutlineOutlinedIcon from "@mui/icons-material/ModeEditOutlineOutlined";
import DeleteOutlinedIcon from "@mui/icons-material/DeleteOutlined";
import VisibilityOutlinedIcon from "@mui/icons-material/VisibilityOutlined";
import Link from "next/link";
import { FieldError, FormError } from "..";
import validator from "validator";
import AddApiKey from "../../components/dashboard/AddApiKey";
import UpdateApiKey from "../../components/dashboard/UpdateApiKey";
import LoadingButton from "@mui/lab/LoadingButton";

export interface APIKeyType {
  id: number;
  title: string;
  key: string;
  secret: string;
  createdAt: string;
  updatedAt: string;
}

const modalStyle = {
  position: "absolute" as "absolute",
  top: "50%",
  left: "50%",
  transform: "translate(-50%, -50%)",
  width: 400,
  bgcolor: "background.paper",
  border: "2px solid #000",
  boxShadow: 24,
  p: 4,
};

export interface ApiKeyForm {
  id: number;
  title: string;
  key: string;
  secret: string;
}

const KeyManagement = () => {
  const { token, userInfo, setNavTitle } = useAuth();
  const [apiKeys, setApiKeys] = useState<Array<APIKeyType>>([]);
  const [openAddModal, setOpenAddModal] = React.useState(false);
  const [openDialog, setOpenDialog] = React.useState(false);
  const [loading, setLoading] = React.useState(false);
  const [initLoading, setInitLoading] = useState<boolean>(true);
  const [errors, setErrors] = useState<Array<FormError>>([]);
  const [selectedId, setSelectedId] = useState<number | null>(null);
  const [formValue, setFormValue] = useState<ApiKeyForm>({
    id: 0,
    title: "",
    key: "",
    secret: "",
  });

  const handleEdit = (item: APIKeyType) => {
    setSelectedId(item.id);
    setFormValue({
      id: item.id,
      title: item.title,
      key: item.key,
      secret: item.secret,
    });
    setOpenAddModal(true);
  };
  const handleDelete = (item: APIKeyType) => {
    setSelectedId(item.id);
    setOpenDialog(true);
  };
  const doDelete = async () => {
    try {
      setLoading(true);
      const response = await axiosPrivate(token!).delete(
        "/api/apiKeys/" + selectedId,
        {
          headers: {
            "Content-Type": "application/json",
          },
        }
      );
      setLoading(false);
      setOpenDialog(false);
      fetchData();
    } catch (e) {
      setLoading(false);
    }
  };

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    setErrors([]);
    const data = new FormData(event.currentTarget);
    let newErrors: Array<FormError> = [];

    if (validator.isEmpty(data.get("title")!.toString())) {
      newErrors.push({ field: "title", error: "Required field" });
    }
    if (validator.isEmpty(data.get("key")!.toString())) {
      newErrors.push({ field: "key", error: "Required field" });
    }
    if (validator.isEmpty(data.get("secret")!.toString())) {
      newErrors.push({ field: "secret", error: "Required field" });
    }

    setErrors(newErrors)
    if (newErrors.length === 0) {
      try {
        setLoading(true);
        if (selectedId == null) {
          // Insert
          const response = await axiosPrivate(token!).post(
            "/api/apiKeys",
            JSON.stringify({
              title: data.get("title")?.toString(),
              key: data.get("key")?.toString(),
              secret: data.get("secret")?.toString(),
            }),
            {
              headers: {
                "Content-Type": "application/json",
              },
            }
          );
        } else {
          // Update
          const response = await axiosPrivate(token!).put(
            "/api/apiKeys/" + selectedId,
            JSON.stringify({
              title: data.get("title")?.toString(),
              key: data.get("key")?.toString(),
              secret: data.get("secret")?.toString(),
            }),
            {
              headers: {
                "Content-Type": "application/json",
              },
            }
          );
        }
        setLoading(false);
        setOpenAddModal(false);
        fetchData();
      } catch (e) {
        setLoading(false);
        const axiosError = e as AxiosError;
        if (axiosError.response) {
          const response = axiosError.response.data as any;
          if (response["error"] == "DUPLICATE_KEY") {
            setErrors((prevErrors) => [
              ...prevErrors,
              { field: "key", error: "Duplicate" },
            ]);
          }
          if (response["error"] == "DUPLICATE_TITLE") {
            setErrors((prevErrors) => [
              ...prevErrors,
              { field: "title", error: "Duplicate" },
            ]);
          }
        }
      }
    }
  };

  const getFieldError = (field: string): FieldError => {
    const find = errors.find((item) => item.field == field);
    if (find == undefined) {
      return {
        valid: false,
        text: "",
      };
    }
    return {
      valid: true,
      text: find.error,
    };
  };

  const columns: GridColDef[] = [
    {
      field: "id",
      headerName: "No.",
      width: 70,
      renderCell: (params) => {
        const index = apiKeys.findIndex((item) => item.id == params.value);
        return index + 1;
      },
    },
    { field: "title", headerName: "Title", width: 130 },
    { field: "key", headerName: "Key", width: 200 },
    {
      field: "secret",
      headerName: "Secret",
      type: "string",
      width: 200,
    },
    {
      field: "createdAt",
      headerName: "Created",
      //description: 'This column has a value getter and is not sortable.',
      sortable: false,
      width: 160,
      //valueGetter: (value: any, row: any) => `${row.firstName || ''} ${row.lastName || ''}`,
    },
    {
      field: "updatedAt",
      headerName: "Updated",
      //description: 'This column has a value getter and is not sortable.',
      sortable: false,
      width: 160,
      //valueGetter: (value: any, row: any) => `${row.firstName || ''} ${row.lastName || ''}`,
    },
    {
      field: "action",
      headerName: "",
      width: 120,
      renderCell: (params) => {
        return (
          <>
            <div
              style={{
                display: "flex",
                alignItems: "center",
                justifyContent: "center",
              }}
            >
              <Link href={`/portfolio/${params.row.id}`}>
                <VisibilityOutlinedIcon />
              </Link>
              <Link href="#" onClick={() => handleEdit(params.row)}>
                <ModeEditOutlineOutlinedIcon />
              </Link>
              <Link href="#" onClick={() => handleDelete(params.row)}>
                <DeleteOutlinedIcon />
              </Link>
            </div>
          </>
        );
      },
    },
  ];

  const fetchData = async () => {
    try {
      if (token) {
        const response = await axiosPrivate(token).get("/api/apiKeys");
        const data: Array<APIKeyType> = response.data;
        setApiKeys(data);
        setInitLoading(false);
      }
    } catch (e) {
      // TODO: Handle errors
      setInitLoading(false);
    }
  };

  useEffect(() => {
    setNavTitle('Key Management');
    fetchData();
  }, [token]);

  return (
    <RequireLogin>
      <DashboardLayout>
        <Grid container spacing={3}>
          {/* Main Content */}
          <Grid item xs={12}>
            <Paper sx={{ p: 2, display: "flex", flexDirection: "column" }}>

            {initLoading ? (
                    <Grid item xs={12} style={{ textAlign: 'center' }}>
                        <p>Loading...</p>
                    </Grid>
                ) : (

                  <>
                <Grid container sx={{ marginBottom: 2 }}>
                  <Button
                    variant="outlined"
                    onClick={() => {
                      setSelectedId(null);
                      setOpenAddModal(true);
                    }}
                  >
                    Add
                  </Button>
                </Grid>

                <div style={{ height: 600, width: "100%" }}>
                  <DataGrid
                    rows={apiKeys}
                    columns={columns}
                    initialState={{
                      pagination: {
                        paginationModel: { page: 0, pageSize: 5 },
                      },
                    }}
                    pageSizeOptions={[5, 10]}
                    checkboxSelection={false}
                    onRowClick={(event, row) => {}}
                  />
                </div>
                  </>

                )}
          

              <Modal
                open={openAddModal}
                onClose={() => {
                  setErrors([]);
                  setOpenAddModal(false);
                }}
                aria-labelledby="modal-modal-title"
                aria-describedby="modal-modal-description"
              >
                <Box sx={modalStyle}>
                  <Box
                    component="form"
                    onSubmit={handleSubmit}
                    noValidate
                    sx={{ mt: 1 }}
                  >
                    {selectedId == null ? (
                      <AddApiKey getFieldError={getFieldError} />
                    ) : (
                      <UpdateApiKey
                        getFieldError={getFieldError}
                        formState={formValue}
                        setFormState={setFormValue}
                      />
                    )}

                    <LoadingButton
                      loading={loading}
                      type="submit"
                      fullWidth
                      variant="contained"
                      sx={{ mt: 3, mb: 2 }}
                    >
                      Save
                    </LoadingButton>
                  </Box>
                </Box>
              </Modal>

              <Dialog
                open={openDialog}
                onClose={() => {}}
                aria-labelledby="alert-dialog-title"
                aria-describedby="alert-dialog-description"
              >
                <DialogTitle id="alert-dialog-title">
                  {"Are you sure?"}
                </DialogTitle>
                <DialogContent>
                  <DialogContentText id="alert-dialog-description">
                    This item will permanently deleted, can not be restored.
                  </DialogContentText>
                </DialogContent>
                <DialogActions>
                  <Button
                    onClick={() => {
                      setOpenDialog(false);
                    }}
                  >
                    Close
                  </Button>
                  <LoadingButton
                    onClick={doDelete}
                    autoFocus
                    color="error"
                    loading={loading}
                  >
                    Delete
                  </LoadingButton>
                </DialogActions>
              </Dialog>
            </Paper>
          </Grid>
        </Grid>
      </DashboardLayout>
    </RequireLogin>
  );
};

export default KeyManagement;

